@@ -205,6 +205,13 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.connect(self.ui.uploadprogressions,
 				QtCore.SIGNAL("activated()"),
 				self.upload_progression)
+			self.connect(self.ui.uploadinstruments,
+				QtCore.SIGNAL("activated()"),
+				self.upload_instruments)
+			self.connect(self.ui.uploadblocks,
+				QtCore.SIGNAL("activated()"),
+				self.upload_blocks)
+
 
 			self.ui.uploadbutton.setEnabled(True)
 			self.ui.menuUpload.setEnabled(True)
@@ -248,7 +255,31 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 		u.show()
 		u.exec_()
 
+	def upload_instruments(self):
+		if self.ui.instruments.count() == 0:
+			q = QtGui.QMessageBox.warning(self, "Error", "No instruments to upload.", 1, 0)
+			return
+		u = UploadDialog(
+			Options.UPLOAD_INSTRUMENTS,
+			self.get_progressions(),
+			self.credentials)
+		u.setWindowTitle("Upload Instruments") 
+		u.ui.uploadname.setText(self.ui.projectname.text())
+		u.show()
+		u.exec_()
 
+	def upload_blocks(self):
+		if self.ui.blocks.count() == 0:
+			q = QtGui.QMessageBox.warning(self, "Error", "No blocks to upload.", 1, 0)
+			return
+		u = UploadDialog(
+			Options.UPLOAD_BLOCKS,
+			self.get_progressions(),
+			self.credentials)
+		u.setWindowTitle("Upload Blocks") 
+		u.ui.uploadname.setText(self.ui.projectname.text())
+		u.show()
+		u.exec_()
 
 	def set_defaults(self):
 
@@ -501,6 +532,12 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.ui.upprogression.setEnabled(enable)
 			self.ui.downprogression.setEnabled(enable)
 
+		if enable and self.ui.instruments.count() > 0:
+			self.ui.uploadsong.setEnabled(enable)
+		else:
+			self.ui.uploadsong.setEnabled(False)
+
+
 	def enable_instruments(self, enable = True):
 		self.ui.exportinstruments.setEnabled(enable)
 		self.ui.uploadinstruments.setEnabled(enable)
@@ -515,6 +552,11 @@ class ImproviserMainWindow(QtGui.QMainWindow):
 			self.ui.upinstrument.setEnabled(enable)
 			self.ui.downinstrument.setEnabled(enable)
 
+		if enable and self.ui.progressions.count() > 0:
+			self.ui.uploadsong.setEnabled(enable)
+		else:
+			self.ui.uploadsong.setEnabled(False)
+
 	def enable_blocks(self, enable = True):
 		self.ui.exportblocks.setEnabled(enable)
 		self.ui.uploadblocks.setEnabled(enable)
